
# q-expansion of newform 150.8.a.h, downloaded from the LMFDB on 13 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 150
weight = 8
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-8], [27], [0], [713], [3810], [-391], [-4182], [-1561], [114150], [-83214], [-83167], [-231334], [-124656], [193757], [319290], [1645428], [-38610], [-1973905], [4409753], [124080], [3967634], [7107992], [8117694], [6727872], [-14268679], [6931188], [13470692], [10562484], [6747965], [730764], [-35396080], [25805940], [10905438], [-53241196], [-33994134], [31340963], [-68348509], [46277687], [-39963900], [-90731154], [130570206], [111280529], [139416246], [-23789269], [46791342], [-116853793], [-183300025], [191649785], [-111119376], [-89221975], [-50467056], [81656316], [-240323383], [-323741124], [-81038484], [211262520], [420832206], [-213858736], [-294664459], [-281001402], [581464739], [-792049002], [203329835], [329427990], [161495459], [639146904], [-539400628], [772336193], [521028426], [522374210], [-1022564418], [-1178962536], [-785348209], [-741463453], [-413198251], [-2077835820], [-1628006664], [654371999], [1917302484], [1465902869], [-1870008780], [1798841150], [36142110], [-1547273413], [-566134303], [951833304], [-1120126764], [2910441158], [459639780], [560050592], [-3707959638], [-3963219546], [1330076351], [-2557849656], [4318553999], [629535528], [5797226406], [1390585998], [1359937715], [615054605], [338340536], [1095768474], [736018566], [908142342], [1021785695], [910202171], [-7805937756], [-5900864760], [-3050054436], [680080883], [7795493888], [-9211680646], [-5268628620], [-5849008189], [2659271717], [497659260], [2596422452], [-4447110828], [-1839713694], [-3415800768], [-4716896926], [-5834301418], [134655042], [-13353565704], [-3266421256], [7829250414], [-3268799143], [-4697654370], [-11050329919], [-15244554874], [-15473758948], [1255503612], [-17510113120], [1558769081], [8145863148], [16193421203], [-3858702816], [-9769495357], [18670364580], [-7979852076], [-17048076049], [-6640275282], [29958613925], [3070557996], [-21044169718], [11604811626], [30801460283], [20344230516], [-5510781976], [25471249008], [-1838847349], [-34071829344], [-25493526643], [22961271030], [5324810732], [22950150702], [-24930907309], [10411522566], [-7281538537], [23027848278], [-46959998082], [34380927912], [11448337928], [185254914], [-30684926766], [-1002874668], [48228215909], [-46883645902], [23864551133], [-2769132558], [51604414482], [52375797659], [-8796201786], [-61214443315], [-5769790321], [-64654801008], [20973260903], [-36516951018], [-29268613951], [52601493371], [41713847189], [34632386808], [47761753745], [-9318855702], [-14193104670], [-12424047834], [-17231957359], [251406791], [10577130029], [-57248325294], [-55319371519], [49089837864], [83519345417], [62835640830], [34014480042], [88039896462], [-113004470650], [59698254158], [15676859610], [-74466711618], [-100311690408], [-12980802412], [-57213606046], [-25931648134], [80348606700], [97327602930], [103968901535], [62687188026], [-51061347300], [-20273410543], [-60449270461], [-1103189478], [-33706839907], [100935365460], [131955821418], [-123451911514], [163150565465], [101859234834], [-2638870848], [-68871032418], [-160152006115], [-32455139605], [-139641716190], [14454798536], [-43131649584], [-98171187517], [42921579762], [-208298023470], [-45389427265], [-18371376822], [144346903109], [-45052251460], [143721126299], [-170654175642], [92072576204], [-109467908712], [-149474764750], [-114987745668], [88280217162], [888985572], [-88710822798], [-97012825855], [152228817791], [-102413819833], [-260658754458], [210639173598], [-36560502661], [139573598514], [-251364433687], [-251038772778], [-26827109986], [-299319691014], [-3660076404], [68882393429], [217890869802], [112909598148], [-64418619379], [29528417636], [42859109796], [129670369166], [7712434685], [-246553964586], [167992915118], [300595628147], [292350301626], [-43098562504], [-28330472550], [277456482072], [195203122268], [66285749898], [43057586411], [263200714760], [352917963818], [269621133845], [-313727700967], [299887365821], [-202038358458], [32243712467], [-468594907777], [188735282640], [239956977384], [-308896490560], [-439290176166], [109028986775], [322386417203], [-298936981206], [-276380057887], [319375766856], [44649444032], [-463406726616], [342424975890], [7761087780], [-453724615812], [-90601479720], [-338559284353], [-298811002728], [-64318648039], [505493646864], [-309519683790], [-304434599644], [-625190982250], [507596312196], [-520267586275], [-177051622926], [702066230708], [577482440597], [-420687449076], [97553450501], [-665405351628], [-285079892722], [144980364234], [431387515884], [104959883196], [-270738029791], [270378886614], [498219256529], [442892377362], [-565725888756], [-533180182495], [-281567005290], [-452932713136], [614636045771], [326226731676], [667430886257], [-436762468062], [-865186964278], [-150083986825], [-126010685497], [-945148193646], [360372464928], [465091735877], [-204177061842], [-351895182787], [1004154580590], [-1075559394817], [-274116640812], [594841965719], [159068135436], [-1031400276091], [-555566933320], [411243013343], [1082341021104], [677537864538], [601946375015], [967072033326], [-454922484954], [1125648220733], [427415626823], [860476840788], [783104814396], [289676027735], [697108440890], [626848102062], [699658088099], [-458501815669], [-424525224282], [388377129408], [-883849147758], [706509484722], [615341273934], [-465968469085], [1198323092262], [-40449897270], [694034466960], [154808442236], [-432465327349], [-997103543400], [-82630106044], [-423171387169], [670326721836], [-471164722879], [-396034402326], [-689774993412], [541796895680], [-949475045911], [-191791828584], [1487995570578], [253049812463], [-1464924322812], [-1674586936627], [1067192249718], [1021491216702], [84026742896], [1726762701504], [-794705022148], [-418373185926], [41032267673], [-670832685922], [124125674564], [2508453858], [-670634900707], [1794554232408], [390191827068], [-2017944307711], [1749723047274], [1166532827201], [-1835896126687], [832912957068], [1424667903545], [1728961932156], [-177790281934], [-2010259692180], [1727587669151], [-327273037068], [9867162420], [-704085577936], [-12692426263], [72727523904], [-284454789673], [2111538122988], [-922541779201], [-723377111172], [162389993490], [-1920368266921], [1830820584221], [-459571995156], [1811583380706], [1537720590431], [439766365344], [-119271027588], [-501669395352], [503815105583], [405779436726], [-500823041838], [844216740974], [1360487863416], [804812967912], [-1760555444124], [1910023274105], [-880068135486]]
